;; Created by Lee Ambrosius (lee_ambrosius@hyperpics.com), HyperPics all rights reserved.
;; Visit HyperPics on the web at: http://www.hyperpics.com
;; Date: 12/23/03

;; Version 1.0 - Works with 2004
;; CustomDWGProps is a command line extension of the Summary Info API or the DWGPROPS command.
;; It allows you to list all fields and also add/remove user defind DWG Properties.  Along
;; with supporting the custom Properties it has full support for the several Standard Properties
;; as well.

;; Bug Watch: The API has a bug that allows Custom Fields to be different based on case.
;;            This doesn't hold true for the UI.
;;
;;            There is also a defect if a value is removed from the UI it still remains
;;            as a blank entry through the API.

(defun c:CustomDWGProps ( / kWord acadObj acDoc acDocSumInfo nCustomProps loopCnt customFieldName fieldValue errHandler)
  (vl-load-com)
  (setq acadObj (vlax-get-acad-object))
  (setq acDoc (vlax-get-property acadObj 'ActiveDocument))
  (setq acDocSumInfo (vlax-get-property acDoc 'SummaryInfo))
  
  (initget "Standard Custom ?")
  (setq kWord (getkword "\nDWG Properties [Standard/Custom/?]: "))
  
  (if (/= kWord nil)
    (progn
 
      (cond
        ((= kWord "?")
          (progn
            ;; Print title
            (prompt "\n\nStandard Properties ")
            (prompt (strcat "\n Title: " (vlax-get-property acDocSumInfo 'Title)))

            ;; Print Subject
            (prompt (strcat "\n Subject: " (vlax-get-property acDocSumInfo 'Subject)))

            ;; Print Author
            (prompt (strcat "\n Author: " (vlax-get-property acDocSumInfo 'Author)))

            ;; Print Keywords
            (prompt (strcat "\n Keywords: " (vlax-get-property acDocSumInfo 'Keywords)))

            ;; Print Comments
            (prompt (strcat "\n Comments: " (vlax-get-property acDocSumInfo 'Comments)))

            ;; Print Hyperlink Base
            (prompt (strcat "\n Hyperlink base: " (vlax-get-property acDocSumInfo 'HyperlinkBase)))

            ;; Get the number of Custom properties
            (setq nCustomProps (vlax-invoke-method acDocSumInfo 'NumCustomInfo))

            ;; Print Last Saved By
            (prompt (strcat "\n Last Saved By: " (vlax-get-property acDocSumInfo 'LastSavedBy)))

            ;; Print Revision Number
            (prompt (strcat "\n Revision Number: " (vlax-get-property acDocSumInfo 'RevisionNumber)))

            ;; Print Custom Properties if there are any 
            (if (= nCustomProps 0)
              (prompt (strcat "\n\nNo Custom Properties\n"))
              (progn
                (setq loopCnt 0)
                (prompt (strcat "\n\nCustom Properties"))
                (repeat nCustomProps
                  (vlax-invoke-method acDocSumInfo 'GetCustomByIndex loopCnt 'customFieldName 'fieldValue)
                  (prompt (strcat "\n " customFieldName ": " fieldValue))
                  (setq loopCnt (1+ loopCnt))
                )
              )
            )
          )
        )

        ((= kWord "Standard")
          (progn
            (initget "Title Author Subject Hyperlinkbase Comments Keywords Lastsavedby Revisionnumber")
            (setq kWord (getkword "\nStandard DWG Properties [Title/Subject/Author/Keywords/Comments/Hyperlinkbase/Lastsavedby/Revisionnumber]: "))

            (if (/= kWord nil)
              (progn
                ;; Get new value for the property
                (setq fieldValue (getstring T "\nEnter value: "))

                ;; Set the new value to the select property
                (cond
                  ((= kWord "Title")(vlax-put-property acDocSumInfo 'Title fieldValue))
                  ((= kWord "Subject")(vlax-put-property acDocSumInfo 'Subject fieldValue))
                  ((= kWord "Author")(vlax-put-property acDocSumInfo 'Author fieldValue))
                  ((= kWord "Keywords")(vlax-put-property acDocSumInfo 'Keywords fieldValue))
                  ((= kWord "Comments")(vlax-put-property acDocSumInfo 'Comments fieldValue))
                  ((= kWord "Hyperlinkbase")(vlax-put-property acDocSumInfo 'Hyperlinkbase fieldValue))
                  ((= kWord "Lastsavedby")(vlax-put-property acDocSumInfo 'Lastsavedby fieldValue))
                  ((= kWord "Revisionnumber")(vlax-put-property acDocSumInfo 'Revisionnumber fieldValue))
                )
              )
            )
          )
        )
      
        ((= kWord "Custom")
          (progn
            (initget "Add Modify Remove")
            (setq kWord (getkword "\nCustom DWG Properties [Add/Modify/Remove]: "))

            (if (/= kWord nil)
              (progn
                (setq customFieldName (getstring T "\nEnter Field Name: "))
   
                (if (or (/= customFieldName "")(/= customFieldName nil))
                  (progn
                    (if (or (= kWord "Add")(= kWord "Modify"))
                      (progn
                        ;; Test to see if the Field is already created or not
                        (setq errHandler (vl-catch-all-apply 'vlax-invoke-method (list acDocSumInfo 'GetCustomByKey customFieldName 'fieldValue)))

                        ;; Get the value for the field
                        (setq fieldValue (getstring T "\nEnter value: "))
      
                        ;; Test error
                        ;;(vl-catch-all-error-message errHandler)
 
                        (if (= errHandler nil)
                          ;;Update extry
                          (vlax-invoke-method acDocSumInfo 'SetCustomByKey customFieldName fieldValue)

                          ;; Field doesn't exist... creating entry
                          (vlax-invoke-method acDocSumInfo 'AddCustomInfo customFieldName fieldValue)
                        )
                      )
                      (progn
                        ;; Test to see if the Field is already created or not
                        (setq errHandler (vl-catch-all-apply 'vlax-invoke-method (list acDocSumInfo 'GetCustomByKey customFieldName 'fieldValue)))
 
                        ;; Test error
                        ;;(vl-catch-all-error-message errHandler)

                        (if (= errHandler nil)
                          ;;Remove extry
                          (vlax-invoke-method acDocSumInfo 'RemoveCustomByKey customFieldName)
                        )
                      )
                    )
                  )
                )
              )
            )
          )
        )
      ) ;; End Cond
    )
  )
 (princ)
)
